/*******************************************************************************
 *	@file			Net.cpp
 *	@brief			Network Library
 *
 *	@author			ohwhsmm7
 *
 *	@date			12-Nov-2011(Sat)
 ******************************************************************************/

// vvZbT---------------------------------------------------------------
#include "Net.h"
#include "TCP.h"
#include "UDP.h"

/*******************************************************************************
 *	@namespace	NET
 ******************************************************************************/
namespace	NET
{

// O[oϐ-----------------------------------------------------------
NET::CLog				g_CLog;													//!< OǗNX
NET::CTCP				g_CTCP;													//!< TCP/IPǗNX
NET::CUDP				g_CUDP;													//!< UDP/IPǗNX

// ֐vg^Cv錾-----------------------------------------------------

// -----------------------------------------------------------------------------
/*******************************************************************************
 *	@fn				void* StartupTCP( const BOOL& bServer,
 *									  const BOOL& bClient,
 *									  const int& nMember,
 *									  const char* cAddr,
 *									  const char* cPort,
 *									  const int& nBufSize,
 *									  const int& nBufAry )
 *	@param[in]		bServer						TCP/IP T[o[gptO
 *	@param[in]		bClient						TCP/IP NCAggptO
 *	@param[in]		nMember						Ql
 *	@param[in]		cAddr						TCP/IP T[o[IPAhX
 *	@param[in]		cPort						TCP/IP T[o[̃|[gԍ
 *	@param[in]		nBufSize					TCP/IPMTCY
 *	@param[in]		nBufAry						m_nBufSize z
 *	@retval			void*						NET::CTCP
 *	@brief			NET::CTCP̐ݒ
 ******************************************************************************/
void* StartupTCP( const BOOL& bServer,
				  const BOOL& bClient,
				  const int& nMember,
				  const char* cAddr,
				  const char* cPort,
				  const int& nBufSize,
				  const int& nBufAry )
{
	assert( ( nMember >= 0 ) && "StartupTCP" );
	assert( ( ::strcmp( cAddr, "" ) != 0 ) && "StartupTCP" );
	assert( ( ::strcmp( cPort, "" ) != 0 ) && "StartupTCP" );

	SNET tcp;
	tcp.m_bServer = bServer;
	tcp.m_bClient = bClient;
	tcp.m_nMemMax = nMember;

	tcp.m_sAddr = cAddr;
	tcp.m_sPort = cPort;
	
	tcp.m_nBufSize = nBufSize;
	tcp.m_nBufAry = nBufAry;
	g_CTCP.Startup( tcp );

	return reinterpret_cast<void*>( &g_CTCP );
}

/*******************************************************************************
 *	@fn				void* StartupUDP( const BOOL& bServer,
 *									  const BOOL& bClient,
 *									  const int& nMember,
 *									  const char* cAddr,
 *									  const char* cPort,
 *									  const int& nBufSize,
 *									  const int& nBufAry )
 *	@param[in]		bServer						UDP/IP T[o[gptO
 *	@param[in]		bClient						UDP/IP NCAggptO
 *	@param[in]		nMember						Ql
 *	@param[in]		cAddr						UDP/IP T[o[IPAhX
 *	@param[in]		cPort						UDP/IP T[o[̃|[gԍ
 *	@param[in]		nBufSize					UDP/IPMTCY
 *	@param[in]		nBufAry						m_nBufSize z
 *	@retval			void*						NET::CUDP
 *	@brief			NET::CUDP̐ݒ
 ******************************************************************************/
void* StartupUDP( const BOOL& bServer,
				  const BOOL& bClient,
				  const int& nMember,
				  const char* cAddr,
				  const char* cPort,
				  const int& nBufSize,
				  const int& nBufAry )
{
	assert( ( nMember >= 0 ) && "StartupUDP" );
	assert( ( ::strcmp( cAddr, "" ) != 0 ) && "StartupUDP" );
	assert( ( ::strcmp( cPort, "" ) != 0 ) && "StartupUDP" );

	SNET udp;
	udp.m_bServer = bServer;
	udp.m_bClient = bClient;
	udp.m_nMemMax = nMember;

	udp.m_sAddr = cAddr;
	udp.m_sPort = cPort;
	
	udp.m_nBufSize = nBufSize;
	udp.m_nBufAry = nBufAry;
	g_CUDP.Startup( udp );

	return reinterpret_cast<void*>( &g_CUDP );
}

/*******************************************************************************
 *	@fn				BOOL Init( void )
 *	@retval			BOOL						TRUE=OK/FALSE=NG
 *	@brief			
 ******************************************************************************/
BOOL Init( void )
{
	WSADATA wsaData;
	// winsock2̏
	if( ::WSAStartup( MAKEWORD( 2, 0 ), &wsaData ) != 0 ) return FALSE;

	// zXg擾
	char hostname[ MAXGETHOSTSTRUCT + 1 ];
	if( ::gethostname( hostname, sizeof( hostname ) ) )
	{
		assert( 0 && "Init : gethostname()" );
		return FALSE;
	}

	// ̂ƂIPv4ňԍŏ̃AhXgp
	char cIP[ 32 ];
	LPHOSTENT host = ::gethostbyname( hostname );
//	for( int i = 0; host->h_addr_list[ i ] != NULL; i++ )
	{
		::sprintf_s( cIP, 32, "%d.%d.%d.%d",
			static_cast<BYTE>( *( ( host->h_addr_list[ 0 ] ) + 0 ) ),
			static_cast<BYTE>( *( ( host->h_addr_list[ 0 ] ) + 1 ) ),
			static_cast<BYTE>( *( ( host->h_addr_list[ 0 ] ) + 2 ) ),
			static_cast<BYTE>( *( ( host->h_addr_list[ 0 ] ) + 3 ) ) );
	}

	// TCP/IP̏
	if( !g_CTCP.Init( &g_CLog, hostname, cIP ) ) return FALSE;
	// UDP/IP̏
	if( !g_CUDP.Init( &g_CLog, hostname, cIP ) ) return FALSE;

	return TRUE;
}

/*******************************************************************************
 *	@fn				void End( void )
 *	@retval			none						Ȃ
 *	@brief			I
 ******************************************************************************/
void End( void )
{
	// UDP/IP̏I
	g_CUDP.End();
	// TCP/IP̏I
	g_CTCP.End();

	// winsock2̏I
	::WSACleanup();
}

/*******************************************************************************
 *	@fn				const std::string GetLog( const int& nNum )
 *	@param[in]		nNum						擾ԍ
 *	@retval			std::string					ObZ[W
 *	@brief			O擾
 ******************************************************************************/
const std::string GetLog( const int& nNum )
{
	return g_CLog.GetLog( nNum );
}

/*******************************************************************************
 *	@fn				const int& GetLogCnt( void )
 *	@retval			int&						OJEg
 *	@brief			OJEg擾
 ******************************************************************************/
const int& GetLogCnt( void )
{
	return g_CLog.GetLogCnt();
}

}	// namespace	NET
